require 'yaml'
require 'pathname'
require 'sequel'

module DbObfuscation
  module Cli
    class Seeder

      def initialize(config_file, path_to_seeds)
        @db_connection = db_connection(config_file)
        files = seed_files(path_to_seeds)
        @seed_data = seed_data(files)
      end

      def self.seed(*args)
        new(*args).seed
      end

      def seed
        @seed_data.each do |table, data|
          data.each do |row|
            @db_connection[table].insert(row)
          end
        end
      end

      private

      def db_connection(config_file)
        config = YAML.load_file(config_file)
        Sequel.connect(config)
      end

      def seed_files(path)
        seeds_path = Pathname.new(path).join('*')
        Dir[seeds_path]
      end

      def table_name(file)
        File.basename(file, '.yml').to_sym
      end

      def seed_data(files)
        files.each_with_object({}) do |file, data|
          table = table_name(file)
          values = YAML.load_file(file).map { |_,v| v }
          data[table] = values
        end
      end
    end
  end
end
