require 'db_obfuscation/environment'
require 'db_obfuscation/database'
require 'db_obfuscation/filtering'
require 'db_obfuscation/query_builder'
require 'db_obfuscation/batch_formulator'

module DbObfuscation
  extend self

  def obfuscate(step)
    config = Filtering.obfuscation_config([:string])
    config.each do |table, cfg|
      DbObfuscation.logging.info "Obfuscating #{table}"
      update(table, cfg, step)
    end
  end

  private

  def update(table, config, step)
    ids(table).each_slice(step) do |ids|
      begin
        batch = BatchFormulator.batch_for(config, ids)
        date_columns = date_columns(config)
        multi_update(table, batch, date_columns) unless batch.empty?
      rescue => e
        DbObfuscation.logging.error 'Encountered Exception'
        DbObfuscation.logging.error "#{table} encountered #{e.message}"
        DbObfuscation.logging.error e.backtrace
      end
    end
  end

  def date_columns(config)
    config.select do |k,v|
      v == :date_strategy
    end.keys
  end

  def ids(table)
    DB[table].map(:id)
  end

  def multi_update(table, batch, date_columns)
    sql_query = QueryBuilder.multi_update_sql(table,
                                              batch,
                                              date_columns)
    DB.run sql_query
  end
end
