require 'yaml'
require 'pathname'

module DbObfuscation
  class Config
    class << self
      def config_path
        @config || DbObfuscation.config_dir
      end

      def config_path=(config)
        @config = Pathname.new(config)
      end

      def db_config
        YAML.load_file(config_path.join('database.yml'))
      end


      def whitelisted_tables
        YAML.load_file(config_path.join('whitelisted_tables.yml'))
      end

      def table_strategies
        @@table_strategies ||= load_table_strategies
      end

      def truncation_patterns
        @truncation_tables ||= YAML.load_file(config_path.join('truncation_patterns.yml')).map(&:to_sym)
      end

      private
      def load_table_strategies
        strategies = {}
        file_path = "#{config_path}/table_strategies/*.yml"
        Dir[file_path].each do |file|
          strategies.merge!(YAML.load_file(file)) if File.file?(file)
        end
        strategies
      end
    end
  end
end
