require 'db_obfuscation/database'
require 'db_obfuscation/filtering/column'
require 'db_obfuscation/truncation'
require 'db_obfuscation/obfuscation_strategy'
require 'active_support/core_ext/object/blank'

module DbObfuscation
  module Filtering
    extend self

    def obfuscation_config(types)
      filter_tables.each_with_object({}) do |table, config|
        table_config = config_per_table(table, types)
        config[table] = table_config if table_config.present?
      end
    end

    private

    def config_per_table(table, types)
      table_config = Column.columns_type(table).each_with_object({}) do |(column_name, column_type), config|
        filtered_column = Column.filter(column_name, column_type, types)
        config[column_name] = default_obfuscation_strategy(filtered_column) if filtered_column
      end.merge(user_config(table))

      reject_whitelisted_columns(table_config)
    end

    def default_obfuscation_strategy(column)
      ObfuscationStrategy.strategy(column)
    end

    def filter_tables
      DbObfuscation::DB.tables - exclude_tables
    end

    def user_config(table)
      table_strategy = DbObfuscation::Config.table_strategies[table.to_s] || {}
      table_strategy.each_with_object({}) { |(k,v), config| config[k.to_sym] = v }
    end

    def reject_whitelisted_columns(config)
      config.reject { |k,v| v == :whitelisted }
    end

    def exclude_tables
      DbObfuscation::Truncation.tables +
      whitelisted_tables +
      [:schema_info]
    end

    def whitelisted_tables
      DbObfuscation::Config.whitelisted_tables.map(&:to_sym)
    end
  end
end
