require 'db_obfuscation/database'

module DbObfuscation
  module Util
    module Trigger
      extend self
      def tables
        query = <<-sql
            SELECT tables.relname as table_name
            FROM pg_trigger triggers, pg_class tables
            WHERE triggers.tgrelid = tables.oid
            AND tables.relname !~ '^pg_'
            AND triggers.tgname LIKE '#{prefix}%'
        sql
        DbObfuscation::DB[query].map(:table_name)
      end

      def drop(table)
        table == :all ? drop_triggers : drop_trigger(table)
      end

      def disable(table)
        table == :all ? disable_triggers : disable_trigger(table)
      end

      def enable(table)
        table == :all ? enable_triggers : enable_trigger(table)
      end

      def exists?(table)
        tables.include?(table.to_s)
      end

      private
      def disable_triggers
        tables.each do |table|
          disable_trigger(table)
        end
      end

      def disable_trigger(table)
        DbObfuscation::DB.run <<-sql
          ALTER TABLE #{table}
          DISABLE TRIGGER #{name(table)}
        sql
      end

      def enable_triggers
        tables.each do |table|
          enable_trigger(table)
        end
      end

      def enable_trigger(table)
        DbObfuscation::DB.run <<-sql
          ALTER TABLE #{table}
          ENABLE TRIGGER #{name(table)}
        sql
      end

      def drop_triggers
        tables.each do |table|
          drop_trigger(table)
        end
      end

      def drop_trigger(table)
        DbObfuscation::DB.run <<-sql
          DROP TRIGGER #{name(table)}
          ON #{table}
        sql
      end

      def prefix
        'audit_'
      end

      def name(table)
        "#{prefix}#{table}"
      end
    end
  end
end
